@@ -1,35 +1,26 @@
 //
-// "$Id: glpuzzle.cxx 4288 2005-04-16 00:13:17Z mike $"
-//
 // OpenGL puzzle demo for the Fast Light Tool Kit (FLTK).
 //
 // This is a GLUT demo program to demonstrate fltk's GLUT emulation.
 // Search for "fltk" to find all the changes
 //
-// Copyright 1998-2005 by Bill Spitzak and others.
+// Copyright 1998-2024 by Bill Spitzak and others.
 //
-// This library is free software; you can redistribute it and/or
-// modify it under the terms of the GNU Library General Public
-// License as published by the Free Software Foundation; either
-// version 2 of the License, or (at your option) any later version.
+// This library is free software. Distribution and use rights are outlined in
+// the file "COPYING" which should have been included with this file.  If this
+// file is missing or damaged, see the license at:
 //
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-// Library General Public License for more details.
+//     https://www.fltk.org/COPYING.php
 //
-// You should have received a copy of the GNU Library General Public
-// License along with this library; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-// USA.
+// Please see the following page on how to report bugs and issues:
 //
-// Please report all bugs and problems on the following page:
-//
-//     http://www.fltk.org/str.php
+//     https://www.fltk.org/bugs.php
 //
 
-// this block added for fltk's distribtion so it will compile w/o OpenGL:
-#include "config.h"
+// Convenience options 'n' and ' ' and command line switch '-n' added for FLTK
+
+// this block added for fltk's distribution so it will compile w/o OpenGL:
+#include <config.h>
 #if !HAVE_GL || !HAVE_GL_GLU_H
 #include <FL/Fl.H>
 #include <FL/fl_message.H>
@@ -46,20 +37,16 @@ int main(int, char**) {
 #include <sys/types.h>
 #include <time.h>
 #include <math.h>
-#include <FL/glut.H>	// changed for fltk
-#ifdef __APPLE__
-# include <OpenGL/glu.h>
-#else
-# include <GL/glu.h> // added for fltk
-#endif
-#include "trackball.c"	// changed from trackball.h for fltk
+#include <FL/glut.H>    // changed for fltk
+#include <FL/glu.h>     // added for fltk
+#include "trackball.c"  // changed from trackball.h for fltk
 
 #define WIDTH 4
 #define HEIGHT 5
 #define PIECES 10
-#define OFFSETX -2
-#define OFFSETY -2.5
-#define OFFSETZ -0.5
+#define OFFSETX -2.0f
+#define OFFSETY -2.5f
+#define OFFSETZ -0.5f
 
 typedef char Config[HEIGHT][WIDTH];
 
@@ -97,6 +84,7 @@ static unsigned char colors[PIECES + 1][3] =
 };
 
 void changeState(void);
+void animate(void);
 
 static struct puzzle *hashtable[HASHSIZE];
 static struct puzzle *startPuzzle;
@@ -105,7 +93,7 @@ static struct puzzlelist *lastentry;
 
 int curX, curY, visible;
 
-#define MOVE_SPEED 0.2
+#define MOVE_SPEED 0.2f
 static unsigned char movingPiece;
 static float move_x, move_y;
 static float curquat[4];
@@ -117,7 +105,7 @@ static char xsize[PIECES + 1] =
 static char ysize[PIECES + 1] =
 {0, 1, 1, 1, 1, 2, 2, 2, 2, 1, 2};
 static float zsize[PIECES + 1] =
-{0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0.6};
+{0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0.6f};
 
 static Config startConfig =
 {
@@ -173,30 +161,30 @@ solution(Config config)
 
 float boxcoords[][3] =
 {
-  {0.2, 0.2, 0.9},
-  {0.8, 0.2, 0.9},
-  {0.8, 0.8, 0.9},
-  {0.2, 0.8, 0.9},
-  {0.2, 0.1, 0.8},
-  {0.8, 0.1, 0.8},
-  {0.9, 0.2, 0.8},
-  {0.9, 0.8, 0.8},
-  {0.8, 0.9, 0.8},
-  {0.2, 0.9, 0.8},
-  {0.1, 0.8, 0.8},
-  {0.1, 0.2, 0.8},
-  {0.2, 0.1, 0.2},
-  {0.8, 0.1, 0.2},
-  {0.9, 0.2, 0.2},
-  {0.9, 0.8, 0.2},
-  {0.8, 0.9, 0.2},
-  {0.2, 0.9, 0.2},
-  {0.1, 0.8, 0.2},
-  {0.1, 0.2, 0.2},
-  {0.2, 0.2, 0.1},
-  {0.8, 0.2, 0.1},
-  {0.8, 0.8, 0.1},
-  {0.2, 0.8, 0.1},
+  {0.2f, 0.2f, 0.9f},
+  {0.8f, 0.2f, 0.9f},
+  {0.8f, 0.8f, 0.9f},
+  {0.2f, 0.8f, 0.9f},
+  {0.2f, 0.1f, 0.8f},
+  {0.8f, 0.1f, 0.8f},
+  {0.9f, 0.2f, 0.8f},
+  {0.9f, 0.8f, 0.8f},
+  {0.8f, 0.9f, 0.8f},
+  {0.2f, 0.9f, 0.8f},
+  {0.1f, 0.8f, 0.8f},
+  {0.1f, 0.2f, 0.8f},
+  {0.2f, 0.1f, 0.2f},
+  {0.8f, 0.1f, 0.2f},
+  {0.9f, 0.2f, 0.2f},
+  {0.9f, 0.8f, 0.2f},
+  {0.8f, 0.9f, 0.2f},
+  {0.2f, 0.9f, 0.2f},
+  {0.1f, 0.8f, 0.2f},
+  {0.1f, 0.2f, 0.2f},
+  {0.2f, 0.2f, 0.1f},
+  {0.8f, 0.2f, 0.1f},
+  {0.8f, 0.8f, 0.1f},
+  {0.2f, 0.8f, 0.1f},
 };
 
 float boxnormals[][3] =
@@ -207,26 +195,26 @@ float boxnormals[][3] =
   {0, 0, -1},
   {0, -1, 0},
   {-1, 0, 0},
-  {0.7071, 0.7071, 0.0000},  /* 6 */
-  {0.7071, -0.7071, 0.0000},
-  {-0.7071, 0.7071, 0.0000},
-  {-0.7071, -0.7071, 0.0000},
-  {0.7071, 0.0000, 0.7071},  /* 10 */
-  {0.7071, 0.0000, -0.7071},
-  {-0.7071, 0.0000, 0.7071},
-  {-0.7071, 0.0000, -0.7071},
-  {0.0000, 0.7071, 0.7071},  /* 14 */
-  {0.0000, 0.7071, -0.7071},
-  {0.0000, -0.7071, 0.7071},
-  {0.0000, -0.7071, -0.7071},
-  {0.5774, 0.5774, 0.5774},  /* 18 */
-  {0.5774, 0.5774, -0.5774},
-  {0.5774, -0.5774, 0.5774},
-  {0.5774, -0.5774, -0.5774},
-  {-0.5774, 0.5774, 0.5774},
-  {-0.5774, 0.5774, -0.5774},
-  {-0.5774, -0.5774, 0.5774},
-  {-0.5774, -0.5774, -0.5774},
+  {0.7071f, 0.7071f, 0.0000f},  /* 6 */
+  {0.7071f, -0.7071f, 0.0000f},
+  {-0.7071f, 0.7071f, 0.0000f},
+  {-0.7071f, -0.7071f, 0.0000f},
+  {0.7071f, 0.0000f, 0.7071f},  /* 10 */
+  {0.7071f, 0.0000f, -0.7071f},
+  {-0.7071f, 0.0000f, 0.7071f},
+  {-0.7071f, 0.0000f, -0.7071f},
+  {0.0000f, 0.7071f, 0.7071f},  /* 14 */
+  {0.0000f, 0.7071f, -0.7071f},
+  {0.0000f, -0.7071f, 0.7071f},
+  {0.0000f, -0.7071f, -0.7071f},
+  {0.5774f, 0.5774f, 0.5774f},  /* 18 */
+  {0.5774f, 0.5774f, -0.5774f},
+  {0.5774f, -0.5774f, 0.5774f},
+  {0.5774f, -0.5774f, -0.5774f},
+  {-0.5774f, 0.5774f, 0.5774f},
+  {-0.5774f, 0.5774f, -0.5774f},
+  {-0.5774f, -0.5774f, 0.5774f},
+  {-0.5774f, -0.5774f, -0.5774f},
 };
 
 int boxfaces[][4] =
@@ -320,30 +308,30 @@ drawBox(int piece, float xoff, float yoff)
 
 float containercoords[][3] =
 {
-  {-0.1, -0.1, 1.0},
-  {-0.1, -0.1, -0.1},
-  {4.1, -0.1, -0.1},
-  {4.1, -0.1, 1.0},
-  {1.0, -0.1, 0.6},     /* 4 */
-  {3.0, -0.1, 0.6},
-  {1.0, -0.1, 0.0},
-  {3.0, -0.1, 0.0},
-  {1.0, 0.0, 0.0},      /* 8 */
-  {3.0, 0.0, 0.0},
-  {3.0, 0.0, 0.6},
-  {1.0, 0.0, 0.6},
-  {0.0, 0.0, 1.0},      /* 12 */
-  {4.0, 0.0, 1.0},
-  {4.0, 0.0, 0.0},
-  {0.0, 0.0, 0.0},
-  {0.0, 5.0, 0.0},      /* 16 */
-  {0.0, 5.0, 1.0},
-  {4.0, 5.0, 1.0},
-  {4.0, 5.0, 0.0},
-  {-0.1, 5.1, -0.1},    /* 20 */
-  {4.1, 5.1, -0.1},
-  {4.1, 5.1, 1.0},
-  {-0.1, 5.1, 1.0},
+  {-0.1f, -0.1f, 1.0f},
+  {-0.1f, -0.1f, -0.1f},
+  {4.1f, -0.1f, -0.1f},
+  {4.1f, -0.1f, 1.0f},
+  {1.0f, -0.1f, 0.6f},     /* 4 */
+  {3.0f, -0.1f, 0.6f},
+  {1.0f, -0.1f, 0.0f},
+  {3.0f, -0.1f, 0.0f},
+  {1.0f, 0.0f, 0.0f},      /* 8 */
+  {3.0f, 0.0f, 0.0f},
+  {3.0f, 0.0f, 0.6f},
+  {1.0f, 0.0f, 0.6f},
+  {0.0f, 0.0f, 1.0f},      /* 12 */
+  {4.0f, 0.0f, 1.0f},
+  {4.0f, 0.0f, 0.0f},
+  {0.0f, 0.0f, 0.0f},
+  {0.0f, 5.0f, 0.0f},      /* 16 */
+  {0.0f, 5.0f, 1.0f},
+  {4.0f, 5.0f, 1.0f},
+  {4.0f, 5.0f, 0.0f},
+  {-0.1f, 5.1f, -0.1f},    /* 20 */
+  {4.1f, 5.1f, -0.1f},
+  {4.1f, 5.1f, 1.0f},
+  {-0.1f, 5.1f, 1.0f},
 };
 
 float containernormals[][3] =
@@ -470,7 +458,7 @@ drawAll(void)
       if (piece == movingPiece) {
         drawBox(piece, move_x, move_y);
       } else {
-        drawBox(piece, j, i);
+        drawBox(piece, float(j), float(i));
       }
     }
   }
@@ -503,7 +491,7 @@ solidifyChain(struct puzzle *puzzle)
     puzzle->backptr->solnptr = puzzle;
     puzzle = puzzle->backptr;
   }
-  sprintf(buf, "%d moves to complete!", i);
+  snprintf(buf, 256, "%d moves to complete!", i);
   glutSetWindowTitle(buf);
 }
 
@@ -725,15 +713,15 @@ continueSolving(void)
       }
     }
   }
-  glutSetWindowTitle("What!  No change?");
+  glutSetWindowTitle((char *)"What!  No change?");
   freeSolutions();
   return 0;
 
 found_piece:
   if (!movingPiece) {
     movingPiece = movedPiece;
-    move_x = fromx;
-    move_y = fromy;
+    move_x = float(fromx);
+    move_y = float(fromy);
   }
   move_x += xadds[movedir] * MOVE_SPEED;
   move_y += yadds[movedir] * MOVE_SPEED;
@@ -759,7 +747,7 @@ solvePuzzle(void)
   int i;
 
   if (solution(thePuzzle)) {
-    glutSetWindowTitle("Puzzle already solved!");
+    glutSetWindowTitle((char *)"Puzzle already solved!");
     return 0;
   }
   addConfig(thePuzzle, NULL);
@@ -775,7 +763,7 @@ solvePuzzle(void)
   }
   if (puzzles == NULL) {
     freeSolutions();
-    sprintf(buf, "I can't solve it! (%d positions examined)", i);
+    snprintf(buf, 256, "I can't solve it! (%d positions examined)", i);
     glutSetWindowTitle(buf);
     return 1;
   }
@@ -878,7 +866,7 @@ int
 invertMatrix(const GLfloat src[16], GLfloat inverse[16])
 {
   int i, j, k, swap;
-  double t;
+  float t;
   GLfloat temp[4][4];
 
   for (i = 0; i < 4; i++) {
@@ -889,7 +877,7 @@ invertMatrix(const GLfloat src[16], GLfloat inverse[16])
   makeIdentity(inverse);
 
   for (i = 0; i < 4; i++) {
-    /* 
+    /*
        ** Look for largest element in column */
     swap = i;
     for (j = i + 1; j < 4; j++) {
@@ -899,7 +887,7 @@ invertMatrix(const GLfloat src[16], GLfloat inverse[16])
     }
 
     if (swap != i) {
-      /* 
+      /*
          ** Swap rows. */
       for (k = 0; k < 4; k++) {
         t = temp[i][k];
@@ -912,7 +900,7 @@ invertMatrix(const GLfloat src[16], GLfloat inverse[16])
       }
     }
     if (temp[i][i] == 0) {
-      /* 
+      /*
          ** No non-zero pivot.  The matrix is singular, which
          shouldn't ** happen.  This means the user gave us a
          bad matrix. */
@@ -938,9 +926,9 @@ invertMatrix(const GLfloat src[16], GLfloat inverse[16])
 
 /*
    ** This is a screwball function.  What it does is the following:
-   ** Given screen x and y coordinates, compute the corresponding object space 
+   ** Given screen x and y coordinates, compute the corresponding object space
    **   x and y coordinates given that the object space z is 0.9 + OFFSETZ.
-   ** Since the tops of (most) pieces are at z = 0.9 + OFFSETZ, we use that 
+   ** Since the tops of (most) pieces are at z = 0.9 + OFFSETZ, we use that
    **   number.
  */
 int
@@ -959,7 +947,7 @@ computeCoords(int piece, int mousex, int mousey,
 
   if (piece == 0)
     return 0;
-  height = zsize[piece] - 0.1 + OFFSETZ;
+  height = zsize[piece] - 0.1f + OFFSETZ;
 
   glGetFloatv(GL_PROJECTION_MATRIX, projMatrix);
   glGetFloatv(GL_MODELVIEW_MATRIX, modelMatrix);
@@ -967,8 +955,8 @@ computeCoords(int piece, int mousex, int mousey,
   if (!invertMatrix(finalMatrix, finalMatrix))
     return 0;
 
-  in[0] = (2.0 * (mousex - viewport[0]) / viewport[2]) - 1;
-  in[1] = (2.0 * ((H - mousey) - viewport[1]) / viewport[3]) - 1;
+  in[0] = (2.0f * (mousex - viewport[0]) / viewport[2]) - 1;
+  in[1] = (2.0f * ((H - mousey) - viewport[1]) / viewport[3]) - 1;
 
   a = in[0] * finalMatrix[0 * 4 + 2] +
     in[1] * finalMatrix[1 * 4 + 2] +
@@ -979,10 +967,10 @@ computeCoords(int piece, int mousex, int mousey,
     finalMatrix[3 * 4 + 3];
   d = finalMatrix[2 * 4 + 3];
 
-  /* 
-     ** Ok, now we need to solve for z: **   (a + b z) / (c + d 
+  /*
+     ** Ok, now we need to solve for z: **   (a + b z) / (c + d
 
-     z) = height. ** ("height" is the height in object space we 
+     z) = height. ** ("height" is the height in object space we
 
      want to solve z for) ** ** ==>  a + b z = height c +
      height d z **      bz - height d z = height c - a ** z =
@@ -994,12 +982,12 @@ computeCoords(int piece, int mousex, int mousey,
 
   z = top / bot;
 
-  /* 
+  /*
      ** Ok, no problem. ** Now we solve for x and y.  We know
      that w = c + d z, so we compute it. */
   w = c + d * z;
 
-  /* 
+  /*
      ** Now for x and y: */
   *selx = (in[0] * finalMatrix[0 * 4 + 0] +
     in[1] * finalMatrix[1 * 4 + 0] +
@@ -1037,8 +1025,8 @@ grabPiece(int piece, float selx, float sely)
     while (selecty > 0 && thePuzzle[selecty - 1][selectx] == movingPiece) {
       selecty--;
     }
-    move_x = selectx;
-    move_y = selecty;
+    move_x = float(selectx);
+    move_y = float(selecty);
     selected = 1;
     selstartx = selx;
     selstarty = sely;
@@ -1106,14 +1094,14 @@ moveSelection(float selx, float sely)
   } else {
     if (deltay > 0 && thePuzzle[selecty][selectx] == 10 &&
       selectx == 1 && selecty == 3) {
-      /* Allow visual movement of solution piece outside of the 
+      /* Allow visual movement of solution piece outside of the
 
          box */
-      move_x = selectx;
+      move_x = float(selectx);
       move_y = sely - selstarty + selecty;
     } else {
-      move_x = selectx;
-      move_y = selecty;
+      move_x = float(selectx);
+      move_y = float(selecty);
     }
   }
 }
@@ -1132,6 +1120,7 @@ static int left_mouse, middle_mouse;
 static int mousex, mousey;
 static int solving;
 static int spinning;
+static int enable_spinning = 1;
 static float lastquat[4];
 static int sel_piece;
 
@@ -1141,7 +1130,7 @@ Reshape(int width, int height)
 
   W = width;
   H = height;
-  glViewport(0, 0, W, H);
+  glViewport(0, 0, (GLsizei)W, (GLsizei)H);
   glGetIntegerv(GL_VIEWPORT, viewport);
 }
 
@@ -1151,12 +1140,12 @@ toggleSolve(void)
     if (solving) {
       freeSolutions();
       solving = 0;
-      glutChangeToMenuEntry(1, "Solving", 1);
-      glutSetWindowTitle("glpuzzle");
+      glutChangeToMenuEntry(1, (char *)"Solving", 1);
+      glutSetWindowTitle((char *)"glpuzzle");
       movingPiece = 0;
     } else {
-      glutChangeToMenuEntry(1, "Stop solving", 1);
-      glutSetWindowTitle("Solving...");
+      glutChangeToMenuEntry(1, (char *)"Stop solving", 1);
+      glutSetWindowTitle((char *)"Solving...");
       if (solvePuzzle()) {
         solving = 1;
       }
@@ -1165,13 +1154,21 @@ toggleSolve(void)
     glutPostRedisplay();
 }
 
+void reset_position(void)
+{
+    spinning = 0;
+    trackball(curquat, 0.0, 0.0, 0.0, 0.0); // reset position
+    glutIdleFunc(animate);
+}
+
 void reset(void)
 {
+    reset_position();
     if (solving) {
       freeSolutions();
       solving = 0;
-      glutChangeToMenuEntry(1, "Solving", 1);
-      glutSetWindowTitle("glpuzzle");
+      glutChangeToMenuEntry(1, (char *)"Solving", 1);
+      glutSetWindowTitle((char *)"glpuzzle");
       movingPiece = 0;
       changeState();
     }
@@ -1188,13 +1185,18 @@ keyboard(unsigned char c, int x, int y)
   case 27:
     exit(0);
     break;
+  case ' ':
+  case 'n':
+  case 'N':
+    reset_position();
+    break;
   case 'D':
   case 'd':
     if (solving) {
       freeSolutions();
       solving = 0;
-      glutChangeToMenuEntry(1, "Solving", 1);
-      glutSetWindowTitle("glpuzzle");
+      glutChangeToMenuEntry(1, (char *)"Solving", 1);
+      glutSetWindowTitle((char *)"glpuzzle");
       movingPiece = 0;
       changeState();
     }
@@ -1235,11 +1237,11 @@ motion(int x, int y)
   if (middle_mouse && !left_mouse) {
     if (mousex != x || mousey != y) {
       trackball(lastquat,
-        (2.0*mousex - W) / W,
-        (H - 2.0*mousey) / H,
-        (2.0*x - W) / W,
-        (H - 2.0*y) / H);
-      spinning = 1;
+        (2.0f*mousex - W) / W,
+        (H - 2.0f*mousey) / H,
+        (2.0f*x - W) / W,
+        (H - 2.0f*y) / H);
+      spinning = enable_spinning; // 1 = yes, 0 = disabled (commandline -n)
     } else {
       spinning = 0;
     }
@@ -1268,8 +1270,8 @@ mouse(int b, int s, int x, int y)
       if (solving) {
         freeSolutions();
         solving = 0;
-      glutChangeToMenuEntry(1, "Solving", 1);
-        glutSetWindowTitle("glpuzzle");
+      glutChangeToMenuEntry(1, (char *)"Solving", 1);
+        glutSetWindowTitle((char *)"glpuzzle");
         movingPiece = 0;
       }
       left_mouse = GL_TRUE;
@@ -1310,11 +1312,11 @@ animate(void)
   if (solving) {
     if (!continueSolving()) {
       solving = 0;
-      glutChangeToMenuEntry(1, "Solving", 1);
-      glutSetWindowTitle("glpuzzle");
+      glutChangeToMenuEntry(1, (char *)"Solving", 1);
+      glutSetWindowTitle((char *)"glpuzzle");
     }
   }
-  if (!solving && !spinning && !visible) {
+  if ((!solving && !spinning) || !visible) {
     glutIdleFunc(NULL);
   }
 }
@@ -1343,11 +1345,11 @@ init(void)
   static float lmodel_local[] =
   {GL_FALSE};
   static float light0_ambient[] =
-  {0.1, 0.1, 0.1, 1.0};
+  {0.1f, 0.1f, 0.1f, 1.0f};
   static float light0_diffuse[] =
-  {1.0, 1.0, 1.0, 0.0};
+  {1.0f, 1.0f, 1.0f, 0.0f};
   static float light0_position[] =
-  {0.8660254, 0.5, 1, 0};
+  {0.8660254f, 0.5f, 1, 0};
   static float light0_specular[] =
   {0.0, 0.0, 0.0, 0.0};
   static float bevel_mat_ambient[] =
@@ -1386,14 +1388,14 @@ init(void)
   glShadeModel(GL_FLAT);
 
   trackball(curquat, 0.0, 0.0, 0.0, 0.0);
-  srandom(time(NULL));
+  srandom((unsigned int)time(NULL));
 }
 
 static void
 Usage(void)
 {
-  printf("Usage: puzzle [-s]\n");
-  printf("   -s:  Run in single buffered mode\n");
+  puts("Usage: puzzle [-s]");
+  puts("   -s:  Run in single buffered mode");
   exit(-1);
 }
 
@@ -1413,12 +1415,15 @@ menu(int choice)
 {
    switch(choice) {
    case 1:
-      toggleSolve();
+      reset_position();
       break;
    case 2:
-      reset();
+      toggleSolve();
       break;
    case 3:
+      reset();
+      break;
+   case 4:
       exit(0);
       break;
    }
@@ -1429,10 +1434,14 @@ main(int argc, char **argv)
 {
   long i;
 
+  Fl::use_high_res_GL(1);
   glutInit(&argc, argv);
   for (i = 1; i < argc; i++) {
     if (argv[i][0] == '-') {
       switch (argv[i][1]) {
+      case 'n':
+        enable_spinning = 0; // disable (sometimes annoying) spinning behaviour
+        break;
       case 's':
         doubleBuffer = 0;
         break;
@@ -1444,7 +1453,7 @@ main(int argc, char **argv)
     }
   }
 
-  glutInitWindowSize(W, H);
+  glutInitWindowSize((int)W, (int)H);
   if (doubleBuffer) {
     glutInitDisplayMode(GLUT_DEPTH | GLUT_RGB | GLUT_DOUBLE | GLUT_MULTISAMPLE);
   } else {
@@ -1458,15 +1467,16 @@ main(int argc, char **argv)
 
   glGetIntegerv(GL_VIEWPORT, viewport);
 
-  printf("\n");
-  printf("r   Reset puzzle\n");
-  printf("s   Solve puzzle (may take a few seconds to compute)\n");
-  printf("d   Destroy a piece - makes the puzzle easier\n");
-  printf("b   Toggles the depth buffer on and off\n");
-  printf("\n");
-  printf("Left mouse moves pieces\n");
-  printf("Middle mouse spins the puzzle\n");
-  printf("Right mouse has menu\n");
+  puts("");
+  puts("n   Normal position - stop spinning");
+  puts("r   Reset puzzle");
+  puts("s   Solve puzzle (may take a few seconds to compute)");
+  puts("d   Destroy a piece - makes the puzzle easier");
+  puts("b   Toggles the depth buffer on and off");
+  puts("");
+  puts("Left mouse moves pieces");
+  puts("Middle mouse spins the puzzle");
+  puts("Right mouse has menu");
 
   glutReshapeFunc(Reshape);
   glutDisplayFunc(redraw);
@@ -1475,16 +1485,13 @@ main(int argc, char **argv)
   glutMouseFunc(mouse);
   glutVisibilityFunc(visibility);
   glutCreateMenu(menu);
-  glutAddMenuEntry("Solve", 1);
-  glutAddMenuEntry("Reset", 2);
-  glutAddMenuEntry("Quit", 3);
+  glutAddMenuEntry("Normal pos", 1);
+  glutAddMenuEntry("Solve", 2);
+  glutAddMenuEntry("Reset", 3);
+  glutAddMenuEntry("Quit", 4);
   glutAttachMenu(GLUT_RIGHT_BUTTON);
   glutMainLoop();
   return 0;             /* ANSI C requires main to return int. */
 }
 
 #endif // added for fltk's distribution
-
-//
-// End of "$Id: glpuzzle.cxx 4288 2005-04-16 00:13:17Z mike $".
-//
